import { Destination as Destination$1, Mapping as Mapping$1, WalkerOS, Hooks, Elb, On } from '@elbwalker/types';
import { SessionConfig } from '@elbwalker/utils/web';

interface Destination<Custom = unknown, CustomEvent = unknown> extends Destination$1.Destination<Custom, CustomEvent> {
    config: Config$1<Custom, CustomEvent>;
    push: Destination$1.PushFn<Custom, CustomEvent>;
    init?: Destination$1.InitFn<Custom, CustomEvent>;
}
type DestinationInit = Destination$1.DestinationInit;
interface Config$1<Custom = unknown, CustomEvent = unknown> extends Destination$1.Config<Custom, CustomEvent> {
}
interface Mapping<CustomEvent = unknown> extends Mapping$1.Config<CustomEvent> {
}
interface EventMapping<CustomEvent = unknown> extends Mapping$1.EventConfig<CustomEvent> {
}

type destination_Destination<Custom = unknown, CustomEvent = unknown> = Destination<Custom, CustomEvent>;
type destination_DestinationInit = DestinationInit;
type destination_EventMapping<CustomEvent = unknown> = EventMapping<CustomEvent>;
type destination_Mapping<CustomEvent = unknown> = Mapping<CustomEvent>;
declare namespace destination {
  export type { Config$1 as Config, destination_Destination as Destination, destination_DestinationInit as DestinationInit, destination_EventMapping as EventMapping, destination_Mapping as Mapping };
}

type Events = Event[];
interface Event {
    entity: string;
    action: string;
    data?: WalkerOS.Properties;
    context?: WalkerOS.OrderedProperties;
    trigger?: Trigger;
    nested: WalkerOS.Entities;
}
type KeyVal = [string, string];
type Attributes = Array<string>;
type Trigger = 'click' | 'custom' | 'hover' | 'load' | 'pulse' | 'scroll' | 'submit' | 'visible' | 'wait' | string;
interface Filter {
    [name: string]: boolean;
}
interface TriggersActionGroups {
    [trigger: string]: TriggerActions;
}
type TriggerActions = Array<TriggerAction>;
interface TriggerAction {
    trigger: string;
    triggerParams?: string;
    action: string;
    actionParams?: string;
}
type ScrollElements = Array<[HTMLElement, number]>;

type walker_Attributes = Attributes;
type walker_Event = Event;
type walker_Events = Events;
type walker_Filter = Filter;
type walker_KeyVal = KeyVal;
type walker_ScrollElements = ScrollElements;
type walker_Trigger = Trigger;
type walker_TriggerActions = TriggerActions;
type walker_TriggersActionGroups = TriggersActionGroups;
declare namespace walker {
  export type { walker_Attributes as Attributes, walker_Event as Event, walker_Events as Events, walker_Filter as Filter, walker_KeyVal as KeyVal, walker_ScrollElements as ScrollElements, walker_Trigger as Trigger, walker_TriggerActions as TriggerActions, walker_TriggersActionGroups as TriggersActionGroups };
}

declare global {
    interface Window {
        elbwalker: Instance;
        walkerjs: Instance;
        elbLayer: Layer;
        dataLayer: WalkerOS.Events | unknown;
        elb: Fn;
    }
}
interface Instance extends WalkerOS.Instance {
    config: Config;
    destinations: Destinations;
    push: Fn;
    getAllEvents: (scope: Element, prefix: string) => Events;
    getEvents: (target: Element, trigger: Trigger, prefix: string) => Events;
    getGlobals: () => WalkerOS.Properties;
    sessionStart: (options?: SessionStartOptions) => void | WalkerOS.SessionData;
}
interface State extends WalkerOS.State {
    config: Config;
    destinations: Destinations;
}
interface Config extends WalkerOS.Config {
    dataLayer: boolean;
    dataLayerConfig: Config$1;
    elbLayer: Layer;
    pageview: boolean;
    prefix: string;
    run: boolean;
    session: false | SessionConfig;
    globalsStatic: WalkerOS.Properties;
    sessionStatic: Partial<WalkerOS.SessionData>;
    elb?: string;
    instance?: string;
}
interface InitConfig extends Partial<Config> {
    consent?: WalkerOS.Consent;
    custom?: WalkerOS.Properties;
    destinations?: Destinations;
    hooks?: Hooks.Functions;
    on?: WalkerOS.OnConfig;
    tagging?: number;
    user?: WalkerOS.User;
}
interface SessionStartOptions {
    config?: SessionConfig;
    data?: Partial<WalkerOS.SessionData>;
}
interface Destinations {
    [name: string]: Destination;
}

type source_Config = Config;
type source_Destinations = Destinations;
type source_InitConfig = InitConfig;
type source_Instance = Instance;
type source_SessionStartOptions = SessionStartOptions;
type source_State = State;
declare namespace source {
  export type { source_Config as Config, source_Destinations as Destinations, source_InitConfig as InitConfig, source_Instance as Instance, source_SessionStartOptions as SessionStartOptions, source_State as State };
}

interface Fn<R = Return, D = PushData, O = PushOptions, C = PushContext> extends Elb.Fn<R, D, O, C>, Arguments<R>, CommandInit<R>, CommandDestination<R>, CommandOn<R>, CommandRun<R> {
}
type Arguments<R = void> = Elb.Arguments<R, PushData, PushOptions, PushContext>;
type CommandInit<R = void> = (event: 'walker init', scope: Scope | Scope[]) => R;
type CommandDestination<R = void> = (event: 'walker destination', destination: Destination | DestinationInit, config?: Config$1) => R;
type CommandRun<R = void> = (event: 'walker run', state?: Partial<State>) => R;
type CommandOn<R = void> = (event: 'walker on', type: 'consent', rules: WalkerOS.SingleOrArray<On.ConsentConfig>) => R;
type PushData = Elb.PushData | Destination | DestinationInit | Partial<State> | ScopeType;
type PushOptions = Elb.PushOptions | Trigger | WalkerOS.SingleOrArray<On.Options> | Config$1;
type PushContext = Elb.PushContext | Element;
type Scope = Element | Document;
type ScopeType = Scope | Scope[];
type PushResult = Elb.PushResult;
type Return<R = Promise<PushResult>> = R;
type Layer = Elb.Layer;

type elb$1_Arguments<R = void> = Arguments<R>;
type elb$1_CommandDestination<R = void> = CommandDestination<R>;
type elb$1_CommandInit<R = void> = CommandInit<R>;
type elb$1_CommandOn<R = void> = CommandOn<R>;
type elb$1_CommandRun<R = void> = CommandRun<R>;
type elb$1_Fn<R = Return, D = PushData, O = PushOptions, C = PushContext> = Fn<R, D, O, C>;
type elb$1_Layer = Layer;
type elb$1_PushContext = PushContext;
type elb$1_PushData = PushData;
type elb$1_PushOptions = PushOptions;
type elb$1_PushResult = PushResult;
type elb$1_Return<R = Promise<PushResult>> = Return<R>;
type elb$1_Scope = Scope;
type elb$1_ScopeType = ScopeType;
declare namespace elb$1 {
  export type { elb$1_Arguments as Arguments, elb$1_CommandDestination as CommandDestination, elb$1_CommandInit as CommandInit, elb$1_CommandOn as CommandOn, elb$1_CommandRun as CommandRun, elb$1_Fn as Fn, elb$1_Layer as Layer, elb$1_PushContext as PushContext, elb$1_PushData as PushData, elb$1_PushOptions as PushOptions, elb$1_PushResult as PushResult, elb$1_Return as Return, elb$1_Scope as Scope, elb$1_ScopeType as ScopeType };
}

declare const elb: Fn;
declare function createSourceWalkerjs(customConfig?: InitConfig): {
    elb: Fn;
    instance: Instance;
};
declare function Walkerjs(customConfig?: InitConfig): Instance;

export { destination as DestinationWeb, elb$1 as Elb, source as SourceWalkerjs, walker as Walker, Walkerjs, createSourceWalkerjs, Walkerjs as default, elb };
